@@ -1,5 +1,5 @@
 /* display.c -- How to display Info windows.
-   $Id: display.c,v 1.3 2004/03/14 00:57:29 karl Exp $
+   $Id: display.c,v 1.4 2004/03/27 15:34:40 karl Exp $
 
    Copyright (C) 1993, 1997, 2003, 2004 Free Software Foundation, Inc.
 
@@ -17,7 +17,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-   Written by Brian Fox (bfox@ai.mit.edu). */
+   Originally written by Brian Fox (bfox@ai.mit.edu). */
 
 #include "info.h"
 #include "display.h"
@@ -52,9 +52,8 @@ void
 display_clear_display (DISPLAY_LINE **display)
 {
   register int i;
-  register DISPLAY_LINE *display_line;
 
-  for (i = 0; (display_line = display[i]); i++)
+  for (i = 0; display[i]; i++)
     {
       display[i]->text[0] = '\0';
       display[i]->textlen = 0;
