@@ -1,7 +1,7 @@
 /* html.h -- declarations for html-related utilities.
-   $Id: html.h,v 1.3 2004/02/13 22:01:38 dirt Exp $
+   $Id: html.h,v 1.4 2004/03/19 18:54:49 karl Exp $
 
-   Copyright (C) 1999, 2000, 2002 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2002, 2004 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -38,7 +38,7 @@ extern int html_title_written;
 extern void html_output_head (void);
 
 /* Escape &<>.  */
-extern char *escape_string (char *string);
+extern char *escape_string (char *);
 
 /* Open or close TAG according to START_OR_END.  */
 extern void insert_html_tag (int start_or_end, char *tag);
