@@ -1,7 +1,8 @@
 /* node.c -- nodes for Texinfo.
-   $Id: node.c,v 1.34 2002/03/26 16:16:29 karl Exp $
+   $Id: node.c,v 1.19 2004/02/29 13:23:51 dirt Exp $
 
-   Copyright (C) 1998, 99, 2000, 01, 02 Free Software Foundation, Inc.
+   Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Free Software
+   Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,6 +21,7 @@
 #include "system.h"
 #include "cmds.h"
 #include "files.h"
+#include "float.h"
 #include "footnote.h"
 #include "macro.h"
 #include "makeinfo.h"
@@ -29,12 +31,12 @@
 #include "insertion.h"
 #include "xml.h"
 
-
 /* See comments in node.h.  */
 NODE_REF *node_references = NULL;
 NODE_REF *node_node_references = NULL;
 TAG_ENTRY *tag_table = NULL;
 int node_number = -1;
+int node_order = 0;
 int current_section = 0;
 int outstanding_node = 0;
 
@@ -42,7 +44,7 @@ int outstanding_node = 0;
 
 /* Start a new tag table. */
 void
-init_tag_table ()
+init_tag_table (void)
 {
   while (tag_table)
     {
@@ -60,8 +62,7 @@ init_tag_table ()
    INDIRECT_P says how to format the output (it depends on whether the
    table is direct or indirect).  */
 static void
-write_tag_table_internal (indirect_p)
-     int indirect_p;
+write_tag_table_internal (int indirect_p)
 {
   TAG_ENTRY *node;
   int old_indent = no_indent;
@@ -111,29 +112,37 @@ write_tag_table_internal (indirect_p)
 }
 
 void
-write_tag_table ()
+write_tag_table (char *filename)
 {
+  output_stream = fopen (filename, "a");
+  if (!output_stream)
+    {
+      fs_error (filename);
+      return;
+    }
+
   write_tag_table_internal (0); /* Not indirect. */
+
+  if (fclose (output_stream) != 0)
+    fs_error (filename);
 }
 
-void
-write_tag_table_indirect ()
+static void
+write_tag_table_indirect (void)
 {
   write_tag_table_internal (1);
 }
 
 /* Convert "top" and friends into "Top". */
 static void
-normalize_node_name (string)
-     char *string;
+normalize_node_name (char *string)
 {
   if (strcasecmp (string, "Top") == 0)
     strcpy (string, "Top");
 }
 
-char *
-get_node_token (expand)
-      int expand;
+static char *
+get_node_token (int expand)
 {
   char *string;
 
@@ -153,8 +162,7 @@ get_node_token (expand)
 /* Expand any macros and other directives in a node name, and
    return the expanded name as an malloc'ed string.  */
 char *
-expand_node_name (node)
-     char *node;
+expand_node_name (char *node)
 {
   char *result = node;
 
@@ -174,8 +182,7 @@ expand_node_name (node)
 /* Look up NAME in the tag table, and return the associated
    tag_entry.  If the node is not in the table return NULL. */
 TAG_ENTRY *
-find_node (name)
-     char *name;
+find_node (char *name)
 {
   TAG_ENTRY *tag = tag_table;
   char *expanded_name;
@@ -222,9 +229,8 @@ find_node (name)
 /* Look in the tag table for a node whose file name is FNAME, and
    return the associated tag_entry.  If there's no such node in the
    table, return NULL. */
-TAG_ENTRY *
-find_node_by_fname (fname)
-     char *fname;
+static TAG_ENTRY *
+find_node_by_fname (char *fname)
 {
   TAG_ENTRY *tag = tag_table;
   while (tag)
@@ -240,8 +246,7 @@ find_node_by_fname (fname)
 /* Remember next, prev, etc. references in a @node command, where we
    don't care about most of the entries. */
 static void
-remember_node_node_reference (node)
-     char *node;
+remember_node_node_reference (char *node)
 {
   NODE_REF *temp = xmalloc (sizeof (NODE_REF));
   int number;
@@ -262,10 +267,9 @@ remember_node_node_reference (node)
 }
 
 /* Remember NODE and associates. */
-void
-remember_node (node, prev, next, up, position, line_no, fname, flags)
-     char *node, *prev, *next, *up, *fname;
-     int position, line_no, flags;
+static void
+remember_node (char *node, char *prev, char *next, char *up,
+    int position, int line_no, char *fname, int flags)
 {
   /* Check for existence of this tag already. */
   if (validating)
@@ -306,8 +310,18 @@ remember_node (node, prev, next, up, position, line_no, fname, flags)
         node_number++;
         new->number = node_number;
       }
-    new->html_fname = fname;
+    if (fname)
+      new->html_fname = fname;
+    else
+      /* This happens for Top node under split-HTML, for example.  */
+      new->html_fname
+	= normalize_filename (filename_part (current_output_filename));
     new->next_ent = tag_table;
+
+    /* Increment the order counter, and save it.  */
+    node_order++;
+    new->order = node_order;
+
     tag_table = new;
   }
 
@@ -324,10 +338,7 @@ remember_node (node, prev, next, up, position, line_no, fname, flags)
    output file has been written, if validation is on, then we use the
    contents of `node_references' as a list of nodes to validate.  */
 void
-remember_node_reference (node, line, type)
-     char *node;
-     int line;
-     enum reftype type;
+remember_node_reference (char *node, int line, enum reftype type)
 {
   NODE_REF *temp = xmalloc (sizeof (NODE_REF));
   int number = number_of_node (node);
@@ -351,8 +362,7 @@ remember_node_reference (node, line, type)
 }
 
 static void
-isolate_nodename (nodename)
-     char *nodename;
+isolate_nodename (char *nodename)
 {
   int i, c;
   int paren_seen, paren;
@@ -411,9 +421,7 @@ isolate_nodename (nodename)
    just before the menu line.  If REMEMBER_REF is zero, REF_TYPE is unused.  */
 #define MENU_STARTER "* "
 char *
-glean_node_from_menu (remember_ref, ref_type)
-     int remember_ref;
-     enum reftype ref_type;
+glean_node_from_menu (int remember_ref, enum reftype ref_type)
 {
   int i, orig_offset = input_text_offset;
   char *nodename;
@@ -467,8 +475,7 @@ glean_node_from_menu (remember_ref, ref_type)
 
 /* Set the name of the current output file.  */
 void
-set_current_output_filename (fname)
-     const char *fname;
+set_current_output_filename (const char *fname)
 {
   if (current_output_filename)
     free (current_output_filename);
@@ -478,11 +485,11 @@ set_current_output_filename (fname)
 /* The order is: nodename, nextnode, prevnode, upnode.
    If all of the NEXT, PREV, and UP fields are empty, they are defaulted.
    You must follow a node command which has those fields defaulted
-   with a sectioning command (e.g. @chapter) giving the "level" of that node.
+   with a sectioning command (e.g., @chapter) giving the "level" of that node.
    It is an error not to do so.
    The defaults come from the menu in this node's parent. */
 void
-cm_node ()
+cm_node (void)
 {
   static long epilogue_len = 0L;
   char *node, *prev, *next, *up;
@@ -504,7 +511,6 @@ cm_node ()
 
   if (!html && !already_outputting_pending_notes)
     {
-      if (!xml)
       close_paragraph ();
       output_pending_notes ();
     }
@@ -580,6 +586,7 @@ cm_node ()
 		epilogue_len = ftell (output_stream) - pos1;
 	      fclose (output_stream);
 	      output_stream = NULL;
+              output_position = 0;
 	      tag = find_node_by_fname (fname_for_this_node);
 	    }
 	  free (fname_for_prev_node);
@@ -589,13 +596,17 @@ cm_node ()
   filling_enabled = indented_fill = 0;
   if (!html || (html && splitting))
     current_footnote_number = 1;
-  
+
   if (verbose_mode)
     printf (_("Formatting node %s...\n"), node);
 
   if (macro_expansion_output_stream && !executing_string)
     remember_itext (input_text, input_text_offset);
 
+  /* Reset the line number in each node for Info output, so that
+     index entries will save the line numbers of parent node.  */
+  node_line_number = 0;
+
   no_indent = 1;
   if (xml)
     {
@@ -603,7 +614,7 @@ cm_node ()
       xml_begin_node ();
       if (!docbook)
 	{
-	  xml_insert_element (NODENAME, START);      
+	  xml_insert_element (NODENAME, START);
 	  if (macro_expansion_output_stream && !executing_string)
 	    me_execute_string (node);
 	  else
@@ -615,6 +626,9 @@ cm_node ()
     }
   else if (!no_headers && !html)
     {
+      /* Emacs Info reader cannot grok indented escape sequence.  */
+      kill_self_indent (-1);
+
       add_word_args ("\037\nFile: %s,  Node: ", pretty_output_filename);
 
       if (macro_expansion_output_stream && !executing_string)
@@ -627,7 +641,7 @@ cm_node ()
   /* Check for defaulting of this node's next, prev, and up fields. */
   defaulting = (*next == 0 && *prev == 0 && *up == 0);
 
-  this_section = what_section (input_text + input_text_offset);
+  this_section = what_section (input_text + input_text_offset, NULL);
 
   /* If we are defaulting, then look at the immediately following
      sectioning command (error if none) to determine the node's
@@ -652,7 +666,7 @@ cm_node ()
               }
 
           line_error
-            (_("Node `%s' requires a sectioning command (e.g. %c%s)"),
+            (_("Node `%s' requires a sectioning command (e.g., %c%s)"),
              node, COMMAND_PREFIX, polite_section_name);
         }
       else
@@ -673,6 +687,8 @@ cm_node ()
 
               int orig_offset, orig_size;
 
+              int bye_offset = search_forward ("\n@bye", input_text_offset);
+
               /* No matter what, make this file point back at `(dir)'. */
               free (up);
               up = xstrdup ("(dir)"); /* html fixxme */
@@ -688,6 +704,7 @@ cm_node ()
 
                   input_text_offset = search_forward ("\n@menu", orig_offset);
                   if (input_text_offset > -1
+                      && (bye_offset > -1 && input_text_offset < bye_offset)
                       && cr_or_whitespace (input_text[input_text_offset + 6]))
                     {
                       char *nodename_from_menu = NULL;
@@ -914,48 +931,61 @@ cm_node ()
 	}
 
       if (splitting || !no_headers)
-        { /* Navigation bar.   The <p> avoids the links area running
-             on with old Lynxen.  */
+        { /* Navigation bar. */
+          add_html_block_elt ("<div class=\"node\">\n");
+          /* The <p> avoids the links area running on with old Lynxen. */
           add_word_args ("<p>%s\n", splitting ? "" : "<hr>");
-          add_word_args ("%s<a name=\"", _("Node:"));
+          add_word ("<a name=\"");
           tem = expand_node_name (node);
           add_anchor_name (tem, 0);
-          add_word_args ("\">%s</a>", tem);
           free (tem);
+          add_word ("\"></a>");
 
           if (next)
             {
               tem = expansion (next, 0);
-	      add_word (",\n");
-	      add_word (_("Next:"));
-	      add_word ("<a rel=next accesskey=n href=\"");
+	      add_word ((char *) _("Next:"));
+              add_word ("&nbsp;");
+              
+	      add_word ("<a rel=\"next\" accesskey=\"n\" href=\"");
 	      add_anchor_name (tem, 1);
+              tem = escape_string (tem);
 	      add_word_args ("\">%s</a>", tem);
+	      
               free (tem);
+
+	      if (prev || up)
+		add_word (",\n");
             }
           if (prev)
             {
               tem = expansion (prev, 0);
-	      add_word (",\n");
-	      add_word (_("Previous:"));
-	      add_word ("<a rel=previous accesskey=p href=\"");
+	      add_word ((char *) _("Previous:"));
+              add_word ("&nbsp;");
+	      add_word ("<a rel=\"previous\" accesskey=\"p\" href=\"");
 	      add_anchor_name (tem, 1);
+              tem = escape_string (tem);
 	      add_word_args ("\">%s</a>", tem);
               free (tem);
+
+	      if (up)
+		add_word (",\n");
             }
           if (up)
             {
               tem = expansion (up, 0);
-	      add_word (",\n");
-	      add_word (_("Up:"));
-	      add_word ("<a rel=up accesskey=u href=\"");
+	      add_word ((char *) _("Up:"));
+              add_word ("&nbsp;");
+	      add_word ("<a rel=\"up\" accesskey=\"u\" href=\"");
 	      add_anchor_name (tem, 1);
+              tem = escape_string (tem);
 	      add_word_args ("\">%s</a>", tem);
               free (tem);
             }
           /* html fixxme: we want a `top' or `contents' link here.  */
 
           add_word_args ("\n%s<br>\n", splitting ? "<hr>" : "");
+      	  add_word ("</div>\n");
         }
     }
   else if (docbook)
@@ -971,7 +1001,7 @@ cm_node ()
       if (prev)
 	{
 	  xml_insert_element (NODEPREV, START);
-	  execute_string ("%s", prev);	    
+	  execute_string ("%s", prev);
 	  xml_insert_element (NODEPREV, END);
 	}
       if (up)
@@ -1022,8 +1052,7 @@ cm_node ()
 
 /* Cross-reference target at an arbitrary spot.  */
 void
-cm_anchor (arg)
-     int arg;
+cm_anchor (int arg)
 {
   char *anchor;
   char *fname_for_anchor = NULL;
@@ -1034,6 +1063,9 @@ cm_anchor (arg)
   /* Parse the anchor text.  */
   anchor = get_xref_token (1);
 
+  /* Force all versions of "top" to be "Top". */
+  normalize_node_name (anchor);
+
   /* In HTML mode, need to actually produce some output.  */
   if (html)
     {
@@ -1156,9 +1188,7 @@ cm_anchor (arg)
 
 /* Find NODE in REF_LIST. */
 static NODE_REF *
-find_node_reference (node, ref_list)
-     char *node;
-     NODE_REF *ref_list;
+find_node_reference (char *node, NODE_REF *ref_list)
 {
   NODE_REF *orig_ref_list = ref_list;
   char *expanded_node;
@@ -1196,7 +1226,7 @@ find_node_reference (node, ref_list)
 }
 
 void
-free_node_references ()
+free_node_references (void)
 {
   NODE_REF *list, *temp;
 
@@ -1214,7 +1244,7 @@ free_node_references ()
 }
 
 void
-free_node_node_references ()
+free_node_node_references (void)
 {
   NODE_REF *list, *temp;
 
@@ -1233,8 +1263,7 @@ free_node_node_references ()
 /* Return the number assigned to a named node in either the tag_table
    or node_references list or zero if no number has been assigned. */
 int
-number_of_node (node)
-     char *node;
+number_of_node (char *node)
 {
   NODE_REF *temp_ref;
   TAG_ENTRY *temp_node = find_node (node);
@@ -1256,10 +1285,7 @@ number_of_node (node)
    Menu, Cross, Next, etc.  */
 
 static int
-validate (tag, line, label)
-     char *tag;
-     int line;
-     char *label;
+validate (char *tag, int line, const char *label)
 {
   TAG_ENTRY *result;
 
@@ -1274,7 +1300,7 @@ validate (tag, line, label)
   if (!result)
     {
       line_number = line;
-      line_error (_("%s reference to nonexistent node `%s'"), label, tag);
+      line_error (_("%s reference to nonexistent node `%s' (perhaps incorrect sectioning?)"), label, tag);
       return 0;
     }
   result->touched++;
@@ -1284,9 +1310,8 @@ validate (tag, line, label)
 /* The strings here are followed in the message by `reference to...' in
    the `validate' routine.  They are only used in messages, thus are
    translated.  */
-static char *
-reftype_type_string (type)
-     enum reftype type;
+static const char *
+reftype_type_string (enum reftype type)
 {
   switch (type)
     {
@@ -1300,8 +1325,7 @@ reftype_type_string (type)
 }
 
 static void
-validate_other_references (ref_list)
-     NODE_REF *ref_list;
+validate_other_references (NODE_REF *ref_list)
 {
   char *old_input_filename = input_filename;
 
@@ -1330,8 +1354,7 @@ validate_other_references (ref_list)
    If the Next is different from the Next of the Up,
    then the Next node must have a Prev pointing at this node. */
 void
-validate_file (tag_table)
-     TAG_ENTRY *tag_table;
+validate_file (TAG_ENTRY *tag_table)
 {
   char *old_input_filename = input_filename;
   TAG_ENTRY *tags = tag_table;
@@ -1370,14 +1393,14 @@ validate_file (tag_table)
                   tem1 = expand_node_name (prev);
                   tem2 = expand_node_name (tags->node);
 
-                  if (STREQ (tem1, tem2))
+                  if (tem1 && tem2 && STREQ (tem1, tem2))
                     you_lose = 0;
                   free (tem1);
                   free (tem2);
                 }
               if (you_lose)
                 {
-                  line_error (_("Next field of node `%s' not pointed to"),
+                  line_error (_("Next field of node `%s' not pointed to (perhaps incorrect sectioning?)"),
                               tags->node);
                   file_line_error (temp_tag->filename, temp_tag->line_no,
 				   _("This node (%s) has the bad Prev"),
@@ -1451,7 +1474,7 @@ validate_file (tag_table)
       if (!tags->up
           && !(tags->flags & TAG_FLAG_ANCHOR)
           && strcasecmp (tags->node, "Top") != 0)
-        line_error (_("`%s' has no Up field"), tags->node);
+        line_error (_("`%s' has no Up field (perhaps incorrect sectioning?)"), tags->node);
       else if (tags->up)
         {
           int valid_p = validate (tags->up, tags->line_no, _("Up"));
@@ -1576,8 +1599,7 @@ validate_file (tag_table)
    This means only anchors follow.  */
 
 static int
-last_node_p (tags)
-     TAG_ENTRY *tags;
+last_node_p (TAG_ENTRY *tags)
 {
   int last = 1;
   while (tags->next_ent) {
@@ -1590,28 +1612,96 @@ last_node_p (tags)
         break;
       }
   }
-  
+
   return last;
 }
 
 
+static char *
+enumerate_filename (char *pathname, char *basename, int number)
+{
+  /* Do we need to generate names of subfiles which don't exceed 8+3 limits? */
+  static const int dos_file_names = !HAVE_LONG_FILENAMES (pathname ? pathname : ".");
+  unsigned name_len = strlen (basename);
+  char *filename = xmalloc (10 + strlen (pathname) + name_len);
+  char *base_filename = xmalloc (10 + name_len);
+
+  sprintf (base_filename, "%s-%d", basename, number);
+
+  if (dos_file_names)
+    {
+      char *dot = strchr (base_filename, '.');
+      unsigned base_len = strlen (base_filename);
+
+      if (dot)
+        { /* Make foobar.i1, .., foobar.i99, foobar.100, ... */
+          dot[1] = 'i';
+          memmove (number <= 99 ? dot + 2 : dot + 1,
+              base_filename + name_len + 1,
+              strlen (base_filename + name_len + 1) + 1);
+        }
+      else if (base_len > 8)
+        {
+          /* Make foobar-1, .., fooba-10, .., foob-100, ... */
+          unsigned numlen = base_len - name_len;
+
+          memmove (base_filename + 8 - numlen, base_filename + name_len, numlen + 1);
+        }
+    }
+
+  sprintf (filename, "%s%s", pathname, base_filename);
+
+  return filename;
+}
+
+/* Remove previously split files, to avoid
+   lingering parts of shrinked documents.  */
+void
+clean_old_split_files (char *filename)
+{
+  char *root_filename = filename_part (filename);
+  char *root_pathname = pathname_part (filename);
+  int i;
+
+  /* We break as soon as we hit an inexistent file,
+     so looping until large numbers is harmless.  */
+  for (i = 1; i < 1000; i++)
+    {
+      struct stat st;
+      char *check_file = enumerate_filename (root_pathname, root_filename, i);
+
+      if (stat (check_file, &st) != 0)
+        break;
+      else if (!S_ISDIR (st.st_mode))
+        {
+          /* Give feedback if requested, removing a file is important.  */
+          if (verbose_mode)
+            printf (_("Removing %s\n"), check_file);
+
+          /* Warn user that we cannot remove the file.  */
+          if (unlink (check_file) != 0)
+            warning (_("Can't remove file `%s': %s"), check_file, strerror (errno));
+        }
+
+      free (check_file);
+    }
+}
+
+
 /* Split large output files into a series of smaller files.  Each file
    is pointed to in the tag table, which then gets written out as the
    original file.  The new files have the same name as the original file
    with a "-num" attached.  SIZE is the largest number of bytes to allow
    in any single split file. */
 void
-split_file (filename, size)
-     char *filename;
-     int size;
+split_file (char *filename, int size)
 {
   char *root_filename, *root_pathname;
-  char *the_file, *filename_part ();
+  char *the_file;
   struct stat fileinfo;
   long file_size;
   char *the_header;
   int header_size;
-  int dos_file_names = 0;       /* if nonzero, don't exceed 8+3 limits */
 
   /* Can only do this to files with tag tables. */
   if (!tag_table)
@@ -1620,8 +1710,8 @@ split_file (filename, size)
   if (size == 0)
     size = DEFAULT_SPLIT_SIZE;
 
-  if ((stat (filename, &fileinfo) != 0) ||
-      (((long) fileinfo.st_size) < SPLIT_SIZE_THRESHOLD))
+  if ((stat (filename, &fileinfo) != 0)
+      || (((long) fileinfo.st_size) < size))
     return;
   file_size = (long) fileinfo.st_size;
 
@@ -1632,9 +1722,6 @@ split_file (filename, size)
   root_filename = filename_part (filename);
   root_pathname = pathname_part (filename);
 
-  /* Do we need to generate names of subfiles which don't exceed 8+3 limits? */
-  dos_file_names = !HAVE_LONG_FILENAMES (root_pathname ? root_pathname : ".");
-
   if (!root_pathname)
     root_pathname = xstrdup ("");
 
@@ -1648,6 +1735,10 @@ split_file (filename, size)
     TAG_ENTRY *tags = tag_table;
     char *indirect_info = NULL;
 
+    /* Maybe we want a Local Variables section.  */
+    char *trailer = info_trailer ();
+    int trailer_len = trailer ? strlen (trailer) : 0;
+
     /* Remember the `header' of this file.  The first tag in the file is
        the bottom of the header; the top of the file is the start. */
     the_header = xmalloc (1 + (header_size = tags->position));
@@ -1733,43 +1824,18 @@ split_file (filename, size)
               write_region:
                 {
                   int fd;
-                  char *split_filename, *split_basename;
-                  unsigned root_len = strlen (root_filename);
-
-                  split_filename = xmalloc (10 + strlen (root_pathname)
-                                            + root_len);
-                  split_basename = xmalloc (10 + root_len);
-                  sprintf (split_basename, "%s-%d", root_filename, which_file);
-                  if (dos_file_names)
-                    {
-                      char *dot = strchr (split_basename, '.');
-                      unsigned base_len = strlen (split_basename);
-
-                      if (dot)
-                        { /* Make foobar.i1, .., foobar.i99, foobar.100, ... */
-                          dot[1] = 'i';
-                          memmove (which_file <= 99 ? dot + 2 : dot + 1,
-                                   split_basename + root_len + 1,
-                                   strlen (split_basename + root_len + 1) + 1);
-                        }
-                      else if (base_len > 8)
-                        {
-                          /* Make foobar-1, .., fooba-10, .., foob-100, ... */
-                          unsigned numlen = base_len - root_len;
-
-                          memmove (split_basename + 8 - numlen,
-                                   split_basename + root_len, numlen + 1);
-                        }
-                    }
-                  sprintf (split_filename, "%s%s", root_pathname,
-                           split_basename);
+                  char *split_filename = enumerate_filename (root_pathname,
+                      root_filename, which_file);
+                  char *split_basename = filename_part (split_filename);
 
                   fd = open (split_filename, O_WRONLY|O_TRUNC|O_CREAT, 0666);
                   if (fd < 0
                       || write (fd, the_header, header_size) != header_size
                       || write (fd, the_file + file_top, file_bot - file_top)
                          != (file_bot - file_top)
-                      || (close (fd)) < 0)
+                      || (trailer_len
+                          && write (fd, trailer, trailer_len) != trailer_len)
+                      || close (fd) < 0)
                     {
                       perror (split_filename);
                       if (fd != -1)
@@ -1814,7 +1880,16 @@ split_file (filename, size)
       /* Inhibit newlines. */
       paragraph_is_open = 0;
 
+      /* Write the indirect tag table.  */
       write_tag_table_indirect ();
+
+      /* preserve local variables in info output.  */
+      if (trailer)
+        {
+          insert_string (trailer);
+          free (trailer);
+        }
+
       fclose (output_stream);
       free (the_header);
       free (the_file);
