@@ -1,5 +1,5 @@
 /* install-info -- create Info directory entry(ies) for an Info file.
-   $Id: install-info.c,v 1.4 2004/03/27 00:24:44 karl Exp $
+   $Id: install-info.c,v 1.8 2004/03/29 12:44:23 karl Exp $
 
    Copyright (C) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Free Software
    Foundation, Inc.
@@ -203,6 +203,60 @@ pfatal_with_name (const char *name)
   char *s = concat ("", strerror (errno), _(" for %s"));
   fatal (s, name, 0);
 }
+
+/* Compare the menu item names in LINE1 (line length LEN1)
+   and LINE2 (line length LEN2).  Return 1 if the item name
+   in LINE1 is less, 0 otherwise.  */
+
+static int
+menu_line_lessp (char *line1, int len1, char *line2, int len2)
+{
+  int minlen = (len1 < len2 ? len1 : len2);
+  int i;
+
+  for (i = 0; i < minlen; i++)
+    {
+      /* If one item name is a prefix of the other,
+         the former one is less.  */
+      if (line1[i] == ':' && line2[i] != ':')
+        return 1;
+      if (line2[i] == ':' && line1[i] != ':')
+        return 0;
+      /* If they both continue and differ, one is less.  */
+      if (line1[i] < line2[i])
+        return 1;
+      if (line1[i] > line2[i])
+        return 0;
+    }
+  /* With a properly formatted dir file,
+     we can only get here if the item names are equal.  */
+  return 0;
+}
+
+/* Compare the menu item names in LINE1 (line length LEN1)
+   and LINE2 (line length LEN2).  Return 1 if the item names are equal,
+   0 otherwise.  */
+
+static int
+menu_line_equal (char *line1, int len1, char *line2, int len2)
+{
+  int minlen = (len1 < len2 ? len1 : len2);
+  int i;
+
+  for (i = 0; i < minlen; i++)
+    {
+      /* If both item names end here, they are equal.  */
+      if (line1[i] == ':' && line2[i] == ':')
+        return 1;
+      /* If they both continue and differ, one is less.  */
+      if (line1[i] != line2[i])
+        return 0;
+    }
+  /* With a properly formatted dir file,
+     we can only get here if the item names are equal.  */
+  return 1;
+}
+
 
 /* Given the full text of a menu entry, null terminated,
    return just the menu item name (copied).  */
@@ -315,6 +369,7 @@ strip_info_suffix (char *fname)
 static int
 menu_item_equal (const char *item, char term_char, const char *name)
 {
+  int ret;
   const char *item_basename = item;
   unsigned name_len = strlen (name);
 
@@ -328,15 +383,16 @@ menu_item_equal (const char *item, char term_char, const char *name)
      filename from the new dir entries, not the filename on the command
      line.  Not worrying about those things right now, though.  --karl,
      26mar04.  */
-  while (*item_basename && !IS_SLASH (*item_basename))
+  while (*item_basename && !IS_SLASH (*item_basename)
+	 && *item_basename != term_char)
     item_basename++;
-  if (! *item_basename)  
+  if (! *item_basename || *item_basename == term_char)
     item_basename = item;  /* no /, use original */
   else
     item_basename++;       /* have /, move past it */
     
   /* First, ITEM must actually match NAME (usually it won't).  */
-  int ret = strncasecmp (item_basename, name, name_len) == 0;
+  ret = strncasecmp (item_basename, name, name_len) == 0;
   if (ret)
     {
       /* Then, `foobar' doesn't match `foo', so be sure we've got all of
@@ -1074,7 +1130,6 @@ main (int argc, char **argv)
   char *compression_program;
   char *infile_sans_info;
   char *infile = 0, *dirfile = 0;
-  unsigned infilelen_sans_info;
 
   /* Record the text of the Info file, as a sequence of characters
      and as a sequence of lines.  */
@@ -1304,7 +1359,6 @@ For more information about these matters, see the files named COPYING.\n"));
       infile_basename--;
 
     infile_sans_info = strip_info_suffix (infile_basename);
-    infilelen_sans_info = strlen (infile_sans_info);
   }
 
   something_deleted
@@ -1450,61 +1504,6 @@ findlines (char *data, int size, int *nlinesp)
   *nlinesp = filled;
   return lines;
 }
-
-/* Compare the menu item names in LINE1 (line length LEN1)
-   and LINE2 (line length LEN2).  Return 1 if the item name
-   in LINE1 is less, 0 otherwise.  */
-
-int
-menu_line_lessp (char *line1, int len1, char *line2, int len2)
-{
-  int minlen = (len1 < len2 ? len1 : len2);
-  int i;
-
-  for (i = 0; i < minlen; i++)
-    {
-      /* If one item name is a prefix of the other,
-         the former one is less.  */
-      if (line1[i] == ':' && line2[i] != ':')
-        return 1;
-      if (line2[i] == ':' && line1[i] != ':')
-        return 0;
-      /* If they both continue and differ, one is less.  */
-      if (line1[i] < line2[i])
-        return 1;
-      if (line1[i] > line2[i])
-        return 0;
-    }
-  /* With a properly formatted dir file,
-     we can only get here if the item names are equal.  */
-  return 0;
-}
-
-/* Compare the menu item names in LINE1 (line length LEN1)
-   and LINE2 (line length LEN2).  Return 1 if the item names are equal,
-   0 otherwise.  */
-
-int
-menu_line_equal (char *line1, int len1, char *line2, int len2)
-{
-  int minlen = (len1 < len2 ? len1 : len2);
-  int i;
-
-  for (i = 0; i < minlen; i++)
-    {
-      /* If both item names end here, they are equal.  */
-      if (line1[i] == ':' && line2[i] == ':')
-        return 1;
-      /* If they both continue and differ, one is less.  */
-      if (line1[i] != line2[i])
-        return 0;
-    }
-  /* With a properly formatted dir file,
-     we can only get here if the item names are equal.  */
-  return 1;
-}
-
-
 
 /* This is the comparison function for qsort for a vector of pointers to
    struct spec_section.  (Have to use const void * as the parameter type
